# -*- coding: utf-8 -*-

"""
***************************************************************************
    B-WaterSmart
    Regional Demand Supply Matching GIS tool (RDSMG)
    frompointlayer.py
    ---------------------
    Date                 : April 2023
    Copyright            : (C) 2023 by IWW Water Centre
    Email                : info at iww-online dot de
***************************************************************************
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
***************************************************************************
"""

from qgis.core import QgsProcessing
from qgis.core import QgsProcessingAlgorithm
from qgis.core import QgsProcessingMultiStepFeedback
from qgis.core import QgsProcessingParameterFeatureSource
from qgis.core import QgsProcessingParameterEnum
from qgis.core import QgsProcessingParameterVectorLayer
from qgis.core import QgsProcessingParameterFile
from qgis.core import QgsProcessingParameterFeatureSink
from qgis.core import QgsProcessingParameterDefinition
from qgis.core import QgsExpression
from qgis.core import QgsProcessingParameterDistance
from qgis.core import QgsProcessingParameterRasterLayer
from qgis.core import QgsProcessingParameterField
import processing


class FromPointlayer(QgsProcessingAlgorithm):
    
    def initAlgorithm(self, config=None):
        # standard parameters
        self.addParameter(QgsProcessingParameterFeatureSource('points_of_interest', 'Points of interest', types=[QgsProcessing.TypeVectorPoint], defaultValue=None))
        self.addParameter(QgsProcessingParameterEnum('quality_select', 'Requested water quality', options=['1 = drinking water quality','2 = at least good water quality (good chemical status, ground and surface water)','3 = at least failing to achieve good water quality (failing to achieve good chemical status, ground and surface water)','4 = not specified (all quality classes, or if chemical status is unknown)'], allowMultiple=False, usesStaticStrings=False, defaultValue=[0]))
        self.addParameter(QgsProcessingParameterVectorLayer('country_polygons', 'Country polygons', optional=True, types=[QgsProcessing.TypeVectorPolygon], defaultValue=None))
        self.addParameter(QgsProcessingParameterField('country_field', 'Country polygons name field', optional=True, type=QgsProcessingParameterField.Any, parentLayerParameterName='country_polygons', allowMultiple=False, defaultValue=None))
        self.addParameter(QgsProcessingParameterVectorLayer('state_polygons', 'State polygons', types=[QgsProcessing.TypeVectorPolygon], defaultValue=None))
        self.addParameter(QgsProcessingParameterVectorLayer('county_polygons', 'County polygons', types=[QgsProcessing.TypeVectorPolygon], defaultValue=None))
        self.addParameter(QgsProcessingParameterVectorLayer('community_polygons', 'Community polygons', optional=True, types=[QgsProcessing.TypeVectorPolygon], defaultValue=None))
        self.addParameter(QgsProcessingParameterVectorLayer('land_use', 'Land use', optional=True, types=[QgsProcessing.TypeVectorPolygon], defaultValue=None))
        self.addParameter(QgsProcessingParameterVectorLayer('water_demand_per_capita', 'Water demand per capita', optional=True, types=[QgsProcessing.TypeVectorPolygon], defaultValue=None))
        self.addParameter(QgsProcessingParameterVectorLayer('total_water_consumption', 'Total water consumption', types=[QgsProcessing.TypeVectorPolygon], defaultValue=None))
        self.addParameter(QgsProcessingParameterVectorLayer('water_consumption_by_sectors', 'Water consumption by sectors', optional=True, types=[QgsProcessing.TypeVectorPolygon], defaultValue=None))
        self.addParameter(QgsProcessingParameterVectorLayer('total_water_production', 'Total water production', types=[QgsProcessing.TypeVectorPolygon], defaultValue=None))
        self.addParameter(QgsProcessingParameterVectorLayer('groundwater_bodies', 'Groundwater bodies', types=[QgsProcessing.TypeVectorPolygon], defaultValue=None))
        self.addParameter(QgsProcessingParameterVectorLayer('groundwater_yield', 'Groundwater yield', optional=True, types=[QgsProcessing.TypeVectorPolygon], defaultValue=None))
        self.addParameter(QgsProcessingParameterField('gwyield_class', 'Groundwater yield class field', optional=True, type=QgsProcessingParameterField.Any, parentLayerParameterName='groundwater_yield', allowMultiple=False, defaultValue=None))
        self.addParameter(QgsProcessingParameterField('gwyield_well', 'Groundwater yield withdrawal well field', optional=True, type=QgsProcessingParameterField.Any, parentLayerParameterName='groundwater_yield', allowMultiple=False, defaultValue=None))
        self.addParameter(QgsProcessingParameterField('gwyield_plant', 'Groundwater yield withdrawal plant field', optional=True, type=QgsProcessingParameterField.Any, parentLayerParameterName='groundwater_yield', allowMultiple=False, defaultValue=None))
        self.addParameter(QgsProcessingParameterVectorLayer('surface_water_bodies', 'Surface water bodies', types=[QgsProcessing.TypeVectorLine], defaultValue=None))
        self.addParameter(QgsProcessingParameterVectorLayer('wwtp', 'Waste water treatment plants', optional=True, types=[QgsProcessing.TypeVectorPoint], defaultValue=None))
        self.addParameter(QgsProcessingParameterRasterLayer('precip', 'Precipitation', optional=True, defaultValue=None))
        
        # advanced parameters
        param = QgsProcessingParameterVectorLayer('ws_association_area', 'Water supplier association area', optional=True, types=[QgsProcessing.TypeVectorPolygon], defaultValue=None)
        param.setFlags(param.flags() | QgsProcessingParameterDefinition.FlagAdvanced)
        self.addParameter(param)
        param = QgsProcessingParameterVectorLayer('nature_reserves', 'Nature reserves', optional=True, types=[QgsProcessing.TypeVectorPolygon], defaultValue=None)
        param.setFlags(param.flags() | QgsProcessingParameterDefinition.FlagAdvanced)
        self.addParameter(param)
        param = QgsProcessingParameterVectorLayer('drinking_water_protect', 'Drinking water protection areas', optional=True, types=[QgsProcessing.TypeVectorPolygon], defaultValue=None)
        param.setFlags(param.flags() | QgsProcessingParameterDefinition.FlagAdvanced)
        self.addParameter(param)
        param = QgsProcessingParameterVectorLayer('roads', 'Roads', optional=True, types=[QgsProcessing.TypeVectorLine], defaultValue=None)
        param.setFlags(param.flags() | QgsProcessingParameterDefinition.FlagAdvanced)
        self.addParameter(param)
        #param = QgsProcessingParameterVectorLayer('drought_indices', 'Drought indices', optional=True, types=[QgsProcessing.TypeVectorPolygon], defaultValue=None)
        #param.setFlags(param.flags() | QgsProcessingParameterDefinition.FlagAdvanced)
        #self.addParameter(param)
        param = QgsProcessingParameterVectorLayer('uwot', 'UWOT results as vector file', optional=True, types=[QgsProcessing.TypeVectorPolygon], defaultValue=None)
        param.setFlags(param.flags() | QgsProcessingParameterDefinition.FlagAdvanced)
        self.addParameter(param)
        param = QgsProcessingParameterField('uwot_field', 'UWOT data field', optional=True, type=QgsProcessingParameterField.Any, parentLayerParameterName='uwot', allowMultiple=False, defaultValue=None)
        param.setFlags(param.flags() | QgsProcessingParameterDefinition.FlagAdvanced)
        self.addParameter(param)
        param = QgsProcessingParameterDistance('search_radius', 'Search radius', parentParameterName='points_of_interest', defaultValue=1000)
        param.setFlags(param.flags() | QgsProcessingParameterDefinition.FlagAdvanced)
        param.setMetadata({'widget_wrapper':{'decimals':1}})
        self.addParameter(param)
        
        # outputs
        self.addParameter(QgsProcessingParameterFeatureSink('rdsmg_output', 'RDSMG output', type=QgsProcessing.TypeVectorAnyGeometry, createByDefault=True, defaultValue=None))

    def processAlgorithm(self, parameters, context, model_feedback):
        # Use a multi-step feedback, so that individual child algorithm progress reports are adjusted for the
        # overall progress through the model
        feedback = QgsProcessingMultiStepFeedback(4, model_feedback)
        results = {}
        outputs = {}
        
        # X/Y-Felder zu Layer hinzufügen
        alg_params = {
            'CRS': parameters['points_of_interest'],
            'INPUT': parameters['points_of_interest'],
            'PREFIX': '',
            'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
        }
        outputs['temp'] = processing.run('native:addxyfields', alg_params, context=context, is_child_algorithm=True)
        
        alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD_NAME':'Qual_Selec',
                'FIELD_TYPE':1,
                'FIELD_LENGTH':3,
                'FIELD_PRECISION':3,
                'FORMULA':parameters['quality_select']+1,
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
        outputs['temp'] = processing.run("native:fieldcalculator", alg_params, context=context, is_child_algorithm=True)

        # Country
        if parameters['country_polygons']:
            alg_params = {
                'DISCARD_NONMATCHING': False,
                'INPUT': outputs['temp']['OUTPUT'],
                'JOIN': parameters['country_polygons'],
                'JOIN_FIELDS': parameters['country_field'],
                'METHOD': 0,  # Separates Objekt für jedes passende Objekt erzeugen (eines-zu-vielen)
                'PREDICATE': [5],  # sind innerhalb
                'PREFIX': '',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
            }
            outputs['temp'] = processing.run('native:joinattributesbylocation', alg_params, context=context, is_child_algorithm=True)
        
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD':parameters['country_field'],
                'NEW_NAME':'Country',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:renametablefield",  alg_params, context=context, is_child_algorithm=True)
        
        # State
        if parameters['state_polygons']:
            alg_params = {
                'DISCARD_NONMATCHING': False,
                'INPUT': outputs['temp']['OUTPUT'],
                'JOIN': parameters['state_polygons'],
                'JOIN_FIELDS': ['Fed_State'],
                'METHOD': 0,  # Separates Objekt für jedes passende Objekt erzeugen (eines-zu-vielen)
                'PREDICATE': [5],  # sind innerhalb
                'PREFIX': 'State_',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
            }
            outputs['temp'] = processing.run('native:joinattributesbylocation', alg_params, context=context, is_child_algorithm=True)
        
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD':'State_Fed_State',
                'NEW_NAME':'State',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:renametablefield",  alg_params, context=context, is_child_algorithm=True)
        
        # County
        if parameters['county_polygons']:
            alg_params = {
                'DISCARD_NONMATCHING': False,
                'INPUT': outputs['temp']['OUTPUT'],
                'JOIN': parameters['county_polygons'],
                'JOIN_FIELDS': ['County'],
                'METHOD': 0,  # Separates Objekt für jedes passende Objekt erzeugen (eines-zu-vielen)
                'PREDICATE': [5],  # sind innerhalb
                'PREFIX': 'County_',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
            }
            outputs['temp'] = processing.run('native:joinattributesbylocation', alg_params, context=context, is_child_algorithm=True)
        
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD':'County_County',
                'NEW_NAME':'County',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:renametablefield",  alg_params, context=context, is_child_algorithm=True)
        
        # Community
        if parameters['community_polygons']:
            alg_params = {
                'DISCARD_NONMATCHING': False,
                'INPUT': outputs['temp']['OUTPUT'],
                'JOIN': parameters['community_polygons'],
                'JOIN_FIELDS': ['name'],
                'METHOD': 0,  # Separates Objekt für jedes passende Objekt erzeugen (eines-zu-vielen)
                'PREDICATE': [5],  # sind innerhalb
                'PREFIX': 'Community_',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
            }
            outputs['temp'] = processing.run('native:joinattributesbylocation', alg_params, context=context, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD':'Community_name',
                'NEW_NAME':'Community',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:renametablefield",  alg_params, context=context, is_child_algorithm=True)
            
        # Land use
        if parameters['land_use']:
            alg_params = {
                'DISCARD_NONMATCHING': False,
                'INPUT': outputs['temp']['OUTPUT'],
                'JOIN': parameters['land_use'],
                'JOIN_FIELDS': ['CLC_Lable2'],
                'METHOD': 0,  # Separates Objekt für jedes passende Objekt erzeugen (eines-zu-vielen)
                'PREDICATE': [5],  # sind innerhalb
                'PREFIX': 'LandUse_',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
            }
            outputs['temp'] = processing.run('native:joinattributesbylocation', alg_params, context=context, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD':'LandUse_CLC_Lable2',
                'NEW_NAME':'LandUse',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:renametablefield",  alg_params, context=context, is_child_algorithm=True)
            
        # Water demand per capita
        if parameters['water_demand_per_capita']:
            alg_params = {
                'DISCARD_NONMATCHING': False,
                'INPUT': outputs['temp']['OUTPUT'],
                'JOIN': parameters['water_demand_per_capita'],
                'JOIN_FIELDS': ['PerCapCons'],
                'METHOD': 0,  # Separates Objekt für jedes passende Objekt erzeugen (eines-zu-vielen)
                'PREDICATE': [5],  # sind innerhalb
                'PREFIX': '',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
            }
            outputs['temp'] = processing.run('native:joinattributesbylocation', alg_params, context=context, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD_NAME':'WatDemCap',
                'FIELD_TYPE':2,
                'FIELD_LENGTH':20,
                'FIELD_PRECISION':0,
                'FORMULA':'CASE WHEN "PerCapCons" < 125 THEN \'below average\' WHEN "PerCapCons" > 130 THEN \'above average\' WHEN 125 <= "PerCapCons" <= 130 THEN \'average\' ELSE NULL END',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:fieldcalculator", alg_params, context=context, is_child_algorithm=True)
                        
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'COLUMN':['PerCapCons'],
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:deletecolumn", alg_params, context=context, is_child_algorithm=True)
            
        # Total water consumption
        if parameters['total_water_consumption']:
            alg_params = {
                'DISCARD_NONMATCHING': False,
                'INPUT': outputs['temp']['OUTPUT'],
                'JOIN': parameters['total_water_consumption'],
                'JOIN_FIELDS': ['tot_ConsM3'],
                'METHOD': 0,  # Separates Objekt für jedes passende Objekt erzeugen (eines-zu-vielen)
                'PREDICATE': [5],  # sind innerhalb
                'PREFIX': 'TotalWaterConsumtion_',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
            }
            outputs['temp'] = processing.run('native:joinattributesbylocation', alg_params, context=context, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD':'TotalWaterConsumtion_tot_ConsM3',
                'NEW_NAME':'TotConsM3',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:renametablefield",  alg_params, context=context, is_child_algorithm=True)
            
        # Water consumption by sectors
        if parameters['water_consumption_by_sectors'] and parameters['total_water_consumption']:
            alg_params = {
                'DISCARD_NONMATCHING': False,
                'INPUT': outputs['temp']['OUTPUT'],
                'JOIN': parameters['water_consumption_by_sectors'],
                'JOIN_FIELDS': ['Dem_house'],
                'METHOD': 0,  # Separates Objekt für jedes passende Objekt erzeugen (eines-zu-vielen)
                'PREDICATE': [5],  # sind innerhalb
                'PREFIX': '',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
            }
            outputs['temp'] = processing.run('native:joinattributesbylocation', alg_params, context=context, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD_NAME':'ConsHouse%',
                'FIELD_TYPE':1, #Integer
                'FIELD_LENGTH':3,
                'FIELD_PRECISION':0,
                'FORMULA':'"Dem_house"/"TotConsM3"*100',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:fieldcalculator", alg_params, context=context, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD_NAME':'ConsIndus%',
                'FIELD_TYPE':1, #Integer
                'FIELD_LENGTH':3,
                'FIELD_PRECISION':0,
                'FORMULA':'("TotConsM3"-"Dem_house")/"TotConsM3"*100',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:fieldcalculator", alg_params, context=context, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'COLUMN':['Dem_house', 'Dem_Indus'],
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:deletecolumn", alg_params, context=context, is_child_algorithm=True)
            
        # Water production and coverage of demand
        if parameters['total_water_production'] and parameters['total_water_consumption']:
            alg_params = {
                'DISCARD_NONMATCHING': False,
                'INPUT': outputs['temp']['OUTPUT'],
                'JOIN': parameters['total_water_production'],
                'JOIN_FIELDS': ['tot_ProdM3'],
                'METHOD': 0,  # Separates Objekt für jedes passende Objekt erzeugen (eines-zu-vielen)
                'PREDICATE': [5],  # sind innerhalb
                'PREFIX': '',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
            }
            outputs['temp'] = processing.run('native:joinattributesbylocation', alg_params, context=context, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD':'tot_ProdM3',
                'NEW_NAME':'TotProdM3',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:renametablefield",  alg_params, context=context, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD_NAME':'AURSC',
                'FIELD_TYPE':1,
                'FIELD_LENGTH':10,
                'FIELD_PRECISION':0,
                'FORMULA':'"TotProdM3"-"TotConsM3"',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:fieldcalculator", alg_params, context=context, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD_NAME':'%AURSC',
                'FIELD_TYPE':1,
                'FIELD_LENGTH':10,
                'FIELD_PRECISION':0,
                'FORMULA':'"TotProdM3"/"TotConsM3"*100',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:fieldcalculator", alg_params, context=context, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD_NAME':'AURCC',
                'FIELD_TYPE':2,
                'FIELD_LENGTH':50,
                'FIELD_PRECISION':0,
                'FORMULA':'CASE WHEN "%AURSC" < 90 THEN \'consumption>production\' WHEN "%AURSC" > 110 THEN \'production>consumption\' WHEN 90 <= "%AURSC" <= 110 THEN concat(\'consumption\',char(8776),\'production\') ELSE NULL END',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:fieldcalculator", alg_params, context=context, is_child_algorithm=True)
            
        # Groundwater bodies
        if parameters['groundwater_bodies']:
            alg_params = {
                'DISCARD_NONMATCHING': False,
                'INPUT': outputs['temp']['OUTPUT'],
                'JOIN': parameters['groundwater_bodies'],
                'JOIN_FIELDS': ['class'],
                'METHOD': 0,  # Separates Objekt für jedes passende Objekt erzeugen (eines-zu-vielen)
                'PREDICATE': [5],  # sind innerhalb
                'PREFIX': 'GW_Quality_',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
            }
            outputs['temp'] = processing.run('native:joinattributesbylocation', alg_params, context=context, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD':'GW_Quality_class',
                'NEW_NAME':'GW_Quality',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:renametablefield", alg_params, context=context, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD_NAME':'GW_T_Costs',
                'FIELD_TYPE':2,
                'FIELD_LENGTH':10,
                'FIELD_PRECISION':0,
                'FORMULA':'CASE WHEN "GW_Quality" - "quality_select" >= 2 THEN \'high\' WHEN "GW_Quality" - "quality_select" = 1 THEN \'medium\' WHEN "GW_Quality" - "quality_select" < 1 THEN \'low\' ELSE NULL END',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:fieldcalculator", alg_params, context=context, is_child_algorithm=True)
            
        # Surface water bodies
        if parameters['surface_water_bodies']:
            alg_params = {
                'DISCARD_NONMATCHING': False,
                'INPUT': outputs['temp']['OUTPUT'],
                'INPUT_2': parameters['surface_water_bodies'],
                'FIELDS_TO_COPY':['WB_name','HeavyMet','Pesticides','Industry','OtherSub','class'],
                'PREFIX': 'SW_',
                'NEIGHBORS':1,
                'MAX_DISTANCE':None,
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
            }
            outputs['temp'] = processing.run('native:joinbynearest', alg_params, context=context, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD':'SW_WB_name',
                'NEW_NAME':'SW_Name',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:renametablefield", alg_params, context=context, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD':'distance',
                'NEW_NAME':'SW_DistanM',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:renametablefield", alg_params, context=context, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD':'SW_Pesticides',
                'NEW_NAME':'SW_Pestici',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:renametablefield", alg_params, context=context, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD':'SW_Industry',
                'NEW_NAME':'SW_Indus',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:renametablefield", alg_params, context=context, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD':'SW_OtherSub',
                'NEW_NAME':'SW_Other',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:renametablefield", alg_params, context=context, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD':'SW_HeavyMet',
                'NEW_NAME':'SW_H_Metal',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:renametablefield", alg_params, context=context, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'COLUMN':['n','feature_x','feature_y','nearest_x','nearest_y'],
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:deletecolumn", alg_params, context=context, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD_NAME':'SW_T_Costs',
                'FIELD_TYPE':2,
                'FIELD_LENGTH':10,
                'FIELD_PRECISION':0,
                'FORMULA':'CASE WHEN "SW_Class" - "quality_select" >= 2 THEN \'high\' WHEN "SW_Class" - "quality_select" = 1 THEN \'medium\' WHEN "SW_Class" - "quality_select" < 1 THEN \'low\' ELSE NULL END',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:fieldcalculator", alg_params, context=context, is_child_algorithm=True)
        
        # Groundwater yield
        if parameters['groundwater_yield']:
            alg_params = {
                'DISCARD_NONMATCHING': False,
                'INPUT': outputs['temp']['OUTPUT'],
                'JOIN': parameters['groundwater_yield'],
                'JOIN_FIELDS': [parameters['gwyield_class'], parameters['gwyield_well'], parameters['gwyield_plant']],
                'METHOD': 0,  # Separates Objekt für jedes passende Objekt erzeugen (eines-zu-vielen)
                'PREDICATE': [5],  # sind innerhalb
                'PREFIX': '',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
            }
            outputs['temp'] = processing.run('native:joinattributesbylocation', alg_params, context=context, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD':parameters['gwyield_class'],
                'NEW_NAME':'GW_Yield',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:renametablefield", alg_params, context=context, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD':parameters['gwyield_well'],
                'NEW_NAME':'GW_Well',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:renametablefield", alg_params, context=context, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD':parameters['gwyield_plant'],
                'NEW_NAME':'GW_Plant',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:renametablefield",  alg_params, context=context, is_child_algorithm=True)
            
        # Water supplier association area
        if parameters['ws_association_area']:
            alg_params = {
                'DISCARD_NONMATCHING': False,
                'INPUT': outputs['temp']['OUTPUT'],
                'JOIN': parameters['ws_association_area'],
                'JOIN_FIELDS': ['area name'],
                'METHOD': 0,  # Separates Objekt für jedes passende Objekt erzeugen (eines-zu-vielen)
                'PREDICATE': [5],  # sind innerhalb
                'PREFIX': 'ws_',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
            }
            outputs['temp'] = processing.run('native:joinattributesbylocation', alg_params, context=context, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD':'ws_area name',
                'NEW_NAME':'SupplyArea',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:renametablefield", alg_params, context=context, is_child_algorithm=True)
        
        # Nature reserves
        if parameters['nature_reserves']:
            alg_params = {
            'INPUT': outputs['temp']['OUTPUT'],
            'INTERSECT': parameters['nature_reserves'],
            'METHOD': 0,  # creating new selection
            'PREDICATE': [6],  # are within
            }
            processing.run('native:selectbylocation', alg_params, context=context, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD_NAME':'NatRes',
                'FIELD_TYPE':2,
                'FIELD_LENGTH':3,
                'FIELD_PRECISION':0,
                'FORMULA':"if(is_selected(), 'yes', 'no')",
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:fieldcalculator", alg_params, context=context, is_child_algorithm=True)
            
        # Drinking water protection areas
        if parameters['drinking_water_protect']:
            alg_params = {
            'INPUT': outputs['temp']['OUTPUT'],
            'INTERSECT': parameters['drinking_water_protect'],
            'METHOD': 0,  # creating new selection
            'PREDICATE': [6],  # are within
            }
            processing.run('native:selectbylocation', alg_params, context=context, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD_NAME':'DrinkProt',
                'FIELD_TYPE':2,
                'FIELD_LENGTH':3,
                'FIELD_PRECISION':0,
                'FORMULA':"if(is_selected(), 'yes', 'no')",
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:fieldcalculator",  alg_params, context=context, is_child_algorithm=True)
            
        # Resource availability query
        if parameters['surface_water_bodies'] and parameters['groundwater_bodies']:
            alg_params = {
                'DISCARD_NONMATCHING': False,
                'INPUT': outputs['temp']['OUTPUT'],
                'INPUT_2': parameters['surface_water_bodies'],
                'FIELDS_TO_COPY':['class'],
                'PREFIX': 'Resource_SW_',
                'NEIGHBORS':1,
                'MAX_DISTANCE':parameters['search_radius'],
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run('native:joinbynearest', alg_params, context=context, is_child_algorithm=True)
            
            alg_params = {
                'DISCARD_NONMATCHING': False,
                'INPUT': outputs['temp']['OUTPUT'],
                'INPUT_2': parameters['groundwater_bodies'],
                'FIELDS_TO_COPY':['class'],
                'PREFIX': 'Resource_GW_',
                'NEIGHBORS':1,
                'MAX_DISTANCE':parameters['search_radius'],
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run('native:joinbynearest', alg_params, context=context, is_child_algorithm=True)
        
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD_NAME':'SW_ReqQual',
                'FIELD_TYPE':2,
                'FIELD_LENGTH':3,
                'FIELD_PRECISION':0,
                'FORMULA':'if("Resource_SW_Class"<="quality_select",\'yes\',\'no\')',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:fieldcalculator", alg_params, context=context, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD_NAME':'GW_ReqQual',
                'FIELD_TYPE':2,
                'FIELD_LENGTH':3,
                'FIELD_PRECISION':0,
                'FORMULA':'if("Resource_GW_class"<="quality_select",\'yes\',\'no\')',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:fieldcalculator", alg_params, context=context, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'COLUMN':['Resource_GW_class','Resource_SW_Class','n','distance','feature_x','feature_y','nearest_x','nearest_y','n_2','distance_2','feature_x_2','feature_y_2','nearest_x_2','nearest_y_2'],
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:deletecolumn", alg_params, context=context, is_child_algorithm=True)
        
        # Waste water treatment plants
        if parameters['wwtp']:
            alg_params = {
                'DISCARD_NONMATCHING': False,
                'INPUT': outputs['temp']['OUTPUT'],
                'INPUT_2': parameters['wwtp'],
                'FIELDS_TO_COPY':['name'],
                'PREFIX': 'WWTP_',
                'NEIGHBORS':1,
                'MAX_DISTANCE':None,
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
            }
            outputs['temp'] = processing.run('native:joinbynearest', alg_params, context=context, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD':'distance',
                'NEW_NAME':'WWTP_DistM',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:renametablefield", alg_params, context=context, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD':'WWTP_name',
                'NEW_NAME':'WWTPName',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:renametablefield", alg_params, context=context, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'COLUMN':['n','feature_x','feature_y','nearest_x','nearest_y'],
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:deletecolumn", alg_params, context=context, is_child_algorithm=True)
        
        # Precipitation
        if parameters['precip']:
            alg_params = {
                'COLUMN_PREFIX': 'PRECIP_',
                'INPUT': outputs['temp']['OUTPUT'],
                'RASTERCOPY': parameters['precip'],
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
            }
            outputs['temp'] = processing.run('native:rastersampling', alg_params, context=context, feedback=feedback, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD':'PRECIP_1',
                'NEW_NAME':'PrecipMM',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:renametablefield", alg_params, context=context, is_child_algorithm=True)
        
        # ROAD obstacles to surface water bodies
        if parameters['surface_water_bodies'] and parameters['roads']:
            alg_params = {
                'DESTINATION': parameters['surface_water_bodies'],
                'DISTANCE': None,
                'METHOD': 0,  # Distance to Nearest Point on feature
                'NEIGHBORS': 1,
                'SOURCE': parameters['points_of_interest'],
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
            }
            outputs['shortestline'] = processing.run('native:shortestline', alg_params, context=context, feedback=feedback, is_child_algorithm=True)
            
            alg_params = {
                'FIELD_NAME': 'obsRoads',
                'GROUP_FIELDS': [''],
                'INPUT': parameters['roads'],
                'MODULUS': 0,
                'SORT_ASCENDING': True,
                'SORT_EXPRESSION': '',
                'SORT_NULLS_FIRST': False,
                'START': 0,
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
            }
            outputs['roads'] = processing.run('native:addautoincrementalfield', alg_params, context=context, feedback=feedback, is_child_algorithm=True)
            
            alg_params = {
                'DISCARD_NONMATCHING': False,
                'INPUT': outputs['shortestline']['OUTPUT'],
                'JOIN': outputs['roads']['OUTPUT'],
                'JOIN_FIELDS': ['obsRoads'],
                'PREDICATE': [0],  # intersect
                'SUMMARIES': [0],  # count
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
            }
            outputs['shortestline'] = processing.run('qgis:joinbylocationsummary', alg_params, context=context, feedback=feedback, is_child_algorithm=True)
        
            alg_params = {
                'DISCARD_NONMATCHING': False,
                'INPUT': outputs['temp']['OUTPUT'],
                'JOIN': outputs['shortestline']['OUTPUT'],
                'JOIN_FIELDS': ['obsRoads_count'],
                'METHOD': 0,  # Create separate feature for each matching feature (one-to-many)
                'PREDICATE': [0],  # intersect
                'PREFIX': '',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
            }
            outputs['temp'] = processing.run('native:joinattributesbylocation', alg_params, context=context, feedback=feedback, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD':'obsRoads_count',
                'NEW_NAME':'Obs_Road',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:renametablefield", alg_params, context=context, is_child_algorithm=True)
            
        # NATURE RESERVE obstacles to surface water bodies
        if parameters['surface_water_bodies'] and parameters['nature_reserves']:
            alg_params = {
                'DESTINATION': parameters['surface_water_bodies'],
                'DISTANCE': None,
                'METHOD': 0,  # Distance to Nearest Point on feature
                'NEIGHBORS': 1,
                'SOURCE': parameters['points_of_interest'],
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
            }
            outputs['shortestline'] = processing.run('native:shortestline', alg_params, context=context, feedback=feedback, is_child_algorithm=True)
            
            alg_params = {
                'FIELD_NAME': 'obsNR',
                'GROUP_FIELDS': [''],
                'INPUT': parameters['nature_reserves'],
                'MODULUS': 0,
                'SORT_ASCENDING': True,
                'SORT_EXPRESSION': '',
                'SORT_NULLS_FIRST': False,
                'START': 0,
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
            }
            outputs['nature_reserves'] = processing.run('native:addautoincrementalfield', alg_params, context=context, feedback=feedback, is_child_algorithm=True)
            
            alg_params = {
                'DISCARD_NONMATCHING': False,
                'INPUT': outputs['shortestline']['OUTPUT'],
                'JOIN': outputs['nature_reserves']['OUTPUT'],
                'JOIN_FIELDS': ['obsNR'],
                'PREDICATE': [0],  # intersect
                'SUMMARIES': [0],  # count
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
            }
            outputs['shortestline'] = processing.run('qgis:joinbylocationsummary', alg_params, context=context, feedback=feedback, is_child_algorithm=True)
        
            alg_params = {
                'DISCARD_NONMATCHING': False,
                'INPUT': outputs['temp']['OUTPUT'],
                'JOIN': outputs['shortestline']['OUTPUT'],
                'JOIN_FIELDS': ['obsNR_count'],
                'METHOD': 0,  # Create separate feature for each matching feature (one-to-many)
                'PREDICATE': [0],  # intersect
                'PREFIX': '',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
            }
            outputs['temp'] = processing.run('native:joinattributesbylocation', alg_params, context=context, feedback=feedback, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD':'obsNR_count',
                'NEW_NAME':'Obs_Nature',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:renametablefield", alg_params, context=context, is_child_algorithm=True)
            
        # DRINKING WATER PROTECTION AREA obstacles to surface water bodies
        if parameters['surface_water_bodies'] and parameters['drinking_water_protect']:
            alg_params = {
                'DESTINATION': parameters['surface_water_bodies'],
                'DISTANCE': None,
                'METHOD': 0,  # Distance to Nearest Point on feature
                'NEIGHBORS': 1,
                'SOURCE': parameters['points_of_interest'],
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
            }
            outputs['shortestline'] = processing.run('native:shortestline', alg_params, context=context, feedback=feedback, is_child_algorithm=True)
            
            alg_params = {
                'FIELD_NAME': 'obsDWPA',
                'GROUP_FIELDS': [''],
                'INPUT': parameters['drinking_water_protect'],
                'MODULUS': 0,
                'SORT_ASCENDING': True,
                'SORT_EXPRESSION': '',
                'SORT_NULLS_FIRST': False,
                'START': 0,
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
            }
            outputs['drinking_water_protect'] = processing.run('native:addautoincrementalfield', alg_params, context=context, feedback=feedback, is_child_algorithm=True)
            
            alg_params = {
                'DISCARD_NONMATCHING': False,
                'INPUT': outputs['shortestline']['OUTPUT'],
                'JOIN': outputs['drinking_water_protect']['OUTPUT'],
                'JOIN_FIELDS': ['obsDWPA'],
                'PREDICATE': [0],  # intersect
                'SUMMARIES': [0],  # count
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
            }
            outputs['shortestline'] = processing.run('qgis:joinbylocationsummary', alg_params, context=context, feedback=feedback, is_child_algorithm=True)
        
            alg_params = {
                'DISCARD_NONMATCHING': False,
                'INPUT': outputs['temp']['OUTPUT'],
                'JOIN': outputs['shortestline']['OUTPUT'],
                'JOIN_FIELDS': ['obsDWPA_count'],
                'METHOD': 0,  # Create separate feature for each matching feature (one-to-many)
                'PREDICATE': [0],  # intersect
                'PREFIX': '',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
            }
            outputs['temp'] = processing.run('native:joinattributesbylocation', alg_params, context=context, feedback=feedback, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD':'obsDWPA_count',
                'NEW_NAME':'Obs_Drink',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:renametablefield", alg_params, context=context, is_child_algorithm=True)
            
        # URBAN FABRIC obstacles to surface water bodies
        if parameters['surface_water_bodies'] and parameters['land_use']:
            alg_params = {
                'DESTINATION': parameters['surface_water_bodies'],
                'DISTANCE': None,
                'METHOD': 0,  # Distance to Nearest Point on feature
                'NEIGHBORS': 1,
                'SOURCE': parameters['points_of_interest'],
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
            }
            outputs['shortestline'] = processing.run('native:shortestline', alg_params, context=context, feedback=feedback, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': parameters['land_use'],
                'FIELD': 'Code_18',
                'OPERATOR': 6,  # starts with
                'VALUE': '11',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
            }
            outputs['land_use'] = processing.run('native:extractbyattribute', alg_params, context=context, feedback=feedback, is_child_algorithm=True)
            
            alg_params = {
                'FIELD_NAME': 'obsUF',
                'GROUP_FIELDS': [''],
                'INPUT': outputs['land_use']['OUTPUT'],
                'MODULUS': 0,
                'SORT_ASCENDING': True,
                'SORT_EXPRESSION': '',
                'SORT_NULLS_FIRST': False,
                'START': 0,
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
            }
            outputs['land_use'] = processing.run('native:addautoincrementalfield', alg_params, context=context, feedback=feedback, is_child_algorithm=True)
            
            alg_params = {
                'DISCARD_NONMATCHING': False,
                'INPUT': outputs['shortestline']['OUTPUT'],
                'JOIN': outputs['land_use']['OUTPUT'],
                'JOIN_FIELDS': ['obsUF'],
                'PREDICATE': [0],  # intersect
                'SUMMARIES': [0],  # count
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
            }
            outputs['shortestline'] = processing.run('qgis:joinbylocationsummary', alg_params, context=context, feedback=feedback, is_child_algorithm=True)
        
            alg_params = {
                'DISCARD_NONMATCHING': False,
                'INPUT': outputs['temp']['OUTPUT'],
                'JOIN': outputs['shortestline']['OUTPUT'],
                'JOIN_FIELDS': ['obsUF_count'],
                'METHOD': 0,  # Create separate feature for each matching feature (one-to-many)
                'PREDICATE': [0],  # intersect
                'PREFIX': '',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
            }
            outputs['temp'] = processing.run('native:joinattributesbylocation', alg_params, context=context, feedback=feedback, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD':'obsUF_count',
                'NEW_NAME':'Obs_Urban',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:renametablefield", alg_params, context=context, is_child_algorithm=True)
            
        # UWOT
        if parameters['uwot']:
            alg_params = {
                'DISCARD_NONMATCHING': False,
                'INPUT': outputs['temp']['OUTPUT'],
                'JOIN': parameters['uwot'],
                'JOIN_FIELDS': [parameters['uwot_field']],
                'METHOD': 0,  # Separates Objekt für jedes passende Objekt erzeugen (eines-zu-vielen)
                'PREDICATE': [5],  # sind innerhalb
                'PREFIX': '',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
            }
            outputs['temp'] = processing.run('native:joinattributesbylocation', alg_params, context=context, is_child_algorithm=True)
            
            alg_params = {
                'INPUT': outputs['temp']['OUTPUT'],
                'FIELD':parameters['uwot_field'],
                'NEW_NAME':'UWOT',
                'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT}
            outputs['temp'] = processing.run("native:renametablefield", alg_params, context=context, is_child_algorithm=True)
            
        # Delete field
        alg_params = {
            'COLUMN': [''],
            'INPUT': outputs['temp']['OUTPUT'],
            'OUTPUT': parameters['rdsmg_output']
        }
        outputs['temp'] = processing.run('native:deletecolumn', alg_params, context=context, feedback=feedback, is_child_algorithm=True)
        
        results['rdsmg_output'] = outputs['temp']['OUTPUT']
        context.layerToLoadOnCompletionDetails(results['rdsmg_output']).name = "RDSMG output"
        return results

    def name(self):
        return 'frompointlayer'

    def displayName(self):
        return 'RDSMG-From point layer'

    def group(self):
        return 'RDSMG'

    def groupId(self):
        return 'rdsmg'

    def shortHelpString(self):
        return """<html><body><p><!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0//EN" "http://www.w3.org/TR/REC-html40/strict.dtd">
<html><head><meta name="qrichtext" content="1" /><style type="text/css">
</style></head><body style=" font-family:'MS Shell Dlg 2'; font-size:8.25pt; font-weight:400; font-style:normal;">
<p style=" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;">This algorithm creates a query of the drinking water demand and production, as well as the available water resources within a user-defined radius from the selected query point(s). </p></body></html></p>
<h2>Input parameters</h2>
<h3>Points of interest</h3>
<p>The point of interest is the point at which the query is generated. A point shape layer with the point(s) to be queried must be used as input.</p>

<h3>Requested water quality</h3>
<p>The water quality classification ist based on the Water Framework Directive (WFD) classification.
1 = drinking water quality
2 = at least good water quality (good chemical status, ground and surface water)
3 = at least failing to achieve good water quality (failing to achieve good chemical status, ground and surface water)
4 = not specified (all quality classes, or if chemical status is unknown)</p>

<h3>Country, Federal state, County, Community</h3>
<p>Country, Federal State, County and Community are used for the geographical classification of the query point. Polygon shape layers are required.</p>

<h3>Land use</h3>
<p>The land use refers to Corine Land Cover 2018 by default. Polygon shape layers are required.</p>

<h3>Water demand per capita</h3>
<p>Water consumption per capita and day in l, divided into three categories:
above average: &gt;130 L/P*d
average: 125-130 L/P*d
below average: &lt;125 L/P*d
Polygon shape layers or tables linked to polygon shape layers are required.</p>

<h3>Total water consumption</h3>
<p>Total annual drinking water consumption in m³
Polygon shape layers or tables linked to polygon shape layers are required.</p>

<h3>Water consumption by sectors</h3>
<p>The annual total water consumption by households and small businesses in m³
Polygon shape layers or tables linked to polygon shape layers are required.</p>

<h3>Total water production</h3>
<p>Total annual drinking water production in m³
Polygon shape layers or tables linked to polygon shape layers are required.</p>

<h3>Groundwaterbodies</h3>
<p>To display the groundwater quality, a polygon shape layer of the groundwater bodies is required, which contains the chemical status according to the Water Framework Directive (WFD).</p>

<h3>Yield of groundwater bodies</h3>
<p>To display the yield of groundwater bodies, a polygon shape layer of the groundwater bodies is required, which contains the yield in classes.</p>

<h3>Surface water bodies</h3>
<p>To display the surface water quality, a line shape layer of the surface water bodies is required, which contains the chemical status according to the Water Framework Directive (WFD).</p>

<h3>WWTP</h3>
<p>The location of wastewater treatment plants (WWTP) are used to determine possible waste water reuse options. A point Shape layer is required.</p>

<h3>Precipitation</h3>
<p>The mean annual precipitation data is used to determine possible rainwater harvesting. Raster data are required.</p>

<h3>Water supplier association area</h3>
<p>The Water supplier association area is used to show whether the query point is in the water supplier association area. A polygon shape layer is required.</p>

<h3>Nature reserves, Drinking water protection areas</h3>
<p>The nature reserve/drinking water protection area data are used to show whether the query point is located in a nature reserve/drinking water protection area or whether a nature reserve/drinking water protection area lies between the query point and the nearest surface water body</p>

<h3>UWOT</h3>
<p>If you have UWOT output data for a particular area, you can use it here to make a forecast for that area in the near future.</p>

<h3>Search radius</h3>
<p>The search radius can be variably adjusted. It specifies the radius in which the search is carried out to determine whether the searched water quality is present in groundwater or surface water. It is set to 1000 m by default. The unit can be adjusted as well.</p>

<h2>Outputs</h2>
<h3>RDSMG output</h3>
<p>The output layer is a point layer that contains the query points and the corresponding data on drinking water demand and supply, as well as on the nearest water resources and all the other optional attributes input.</p>
<!-- <br><p align="right">author: author name</p><p align="right">version: 0.1</p></body></html> -->"""

    def createInstance(self):
        return FromPointlayer()
